import { toNumberWithDefaultZero } from "@applicaster/zapp-react-native-utils/numberUtils";
import * as React from "react";
import { ImageStyle, StyleSheet, View, ViewStyle } from "react-native";

type BorderPosition = "inside" | "outside" | "center";

interface Props {
  style: ImageStyle | ViewStyle;
  borderPosition: BorderPosition;
  children: React.ReactNode;
}

const MARGINS = [
  "marginTop",
  "marginBottom",
  "marginLeft",
  "marginRight",
] as const;

const PADDINGS = [
  "paddingTop",
  "paddingBottom",
  "paddingLeft",
  "paddingRight",
] as const;

const isPercentageValue = (value: number | string) => {
  if (typeof value === "string" && value.endsWith("%")) {
    return true;
  }

  return false;
};

const calculateContainerSpacing = ({
  spacingKey,
  style,
  additionalSpacing,
}: {
  spacingKey: typeof MARGINS[number] | typeof PADDINGS[number];
  style: ImageStyle | ViewStyle;
  additionalSpacing: number;
}) => {
  return {
    [spacingKey]:
      toNumberWithDefaultZero(style[spacingKey]) + additionalSpacing,
  };
};

const getContainerBorderWidth = (
  borderPosition: BorderPosition,
  borderWidth
) => {
  switch (borderPosition) {
    case "inside":
      return 0;
    case "center":
      return borderWidth / 2;
    case "outside":
      return borderWidth;
    default:
      return borderWidth;
  }
};

const getContainerPadding = (borderPosition: BorderPosition, borderWidth) => {
  switch (borderPosition) {
    case "inside":
      return borderWidth;
    case "center":
      return borderWidth / 2;
    case "outside":
      return 0;
    default:
      return 0;
  }
};

const InsideBorder = ({
  style,
  borderPosition,
}: {
  style: ViewStyle;
  borderPosition: BorderPosition;
}) => {
  if (borderPosition === "outside") {
    return null;
  }

  const insideBorderWidth =
    borderPosition === "center" ? style.borderWidth / 2 : style.borderWidth;

  return (
    <View
      style={{
        ...StyleSheet.absoluteFillObject,
        borderWidth: insideBorderWidth,
        borderColor: style.borderColor,
        borderRadius: style.borderRadius,
      }}
    />
  );
};

export const BorderContainerView = (props: Props) => {
  const { style, children } = props;
  // Return border position prop after re-implementing
  const borderPosition = "outside";

  const borderWidth = toNumberWithDefaultZero(style.borderWidth);
  const spacing = getContainerPadding(borderPosition, borderWidth);

  const borderStyleForMissingBorder =
    style?.borderWidth === 0 || !style?.borderColor ? { borderRadius: 0 } : {};

  const updatedMargins = MARGINS.reduce((acc, margin) => {
    const marginValue = calculateContainerSpacing({
      spacingKey: margin,
      style,
      additionalSpacing: spacing,
    });

    return { ...acc, ...marginValue };
  }, {});

  const updatedPaddings = PADDINGS.reduce((acc, padding) => {
    const paddingValue = calculateContainerSpacing({
      spacingKey: padding,
      style,
      additionalSpacing: spacing,
    });

    return { ...acc, ...paddingValue };
  }, {});

  const updatedWidth = style.width
    ? isPercentageValue(style.width)
      ? style.width
      : toNumberWithDefaultZero(style.width) - spacing * 2
    : undefined;

  const updatedHeight = style.height
    ? isPercentageValue(style.height)
      ? style.height
      : toNumberWithDefaultZero(style.height) - spacing * 2
    : undefined;

  const containerStyle = {
    borderWidth: getContainerBorderWidth(borderPosition, borderWidth),
    padding: spacing,
    ...updatedMargins,
    ...updatedPaddings,
    width: updatedWidth,
    height: updatedHeight,
  };

  return (
    <View {...props} style={[style, containerStyle]}>
      {children}
      <InsideBorder
        style={{ ...style, ...borderStyleForMissingBorder }}
        borderPosition={borderPosition}
      />
    </View>
  );
};
