/// <reference types="@applicaster/applicaster-types" />
import React from "react";
import * as R from "ramda";
import { View } from "react-native";
import { usePickFromState } from "@applicaster/zapp-react-native-redux/hooks";

import { useTheme } from "@applicaster/zapp-react-native-utils/theme";
import { getComponentModule } from "@applicaster/zapp-react-native-utils/pluginUtils";
import { getNavigationPropsV2 } from "@applicaster/zapp-react-native-utils/navigationUtils";
import { useSafeAreaInsets } from "react-native-safe-area-context";
import {
  useDimensions,
  useRoute,
  useCurrentScreenData,
  useNavbarState,
  useScreenData,
  useNavigation,
} from "@applicaster/zapp-react-native-utils/reactHooks";
import { getNavigationPluginModule } from "@applicaster/zapp-react-native-app/App/Layout/layoutHelpers";

import { ScreenContextProvider } from "../../Contexts/ScreenContext";
import { RouteManager } from "../RouteManager";
import { useScreenConfiguration } from "../River/useScreenConfiguration";
import { useScreenOrientationHandler } from "./orientationHandler";

function colorIsNotTransparent(color) {
  const layers = R.compose(
    R.map(R.trim),
    R.split(","),
    R.replace(")", ""),
    R.replace("rgba(", "")
  )(color);

  const alpha = Number(layers?.[3]);

  return alpha > 0;
}

function isValidColor(string) {
  return string && string !== "transparent" && colorIsNotTransparent(string);
}

type Props = {
  pathname: string;
  screenData: Record<string, any>;
  setToggleMenu: () => void;
  setCloseMenu: () => void;
};

export function Screen({
  setToggleMenu,
  setCloseMenu,
  screenData: _screenData,
}: Props) {
  const theme = useTheme<BaseThemePropertiesMobile>();
  const navigation = useNavigation();
  const { plugins } = usePickFromState(["plugins"]);

  // Gets the data for the current screen configuration
  const currentScreenData = useCurrentScreenData();
  const { backgroundColor } = useScreenConfiguration(currentScreenData.id);
  const insets = useSafeAreaInsets();

  const { width: screenWidth } = useDimensions("screen", {
    fullDimensions: true,
  });

  const routeData = useRoute();

  const currentRiver = useScreenData(
    (routeData.screenData as ZappRiver)?.targetScreen
      ? (routeData.screenData as ZappRiver)?.targetScreen?.id ??
          navigation.activeRiver.id
      : (routeData.screenData as ZappRiver)?.screen_id
  );

  const { title } = useNavbarState();

  const navBarProps = React.useMemo(
    () => ({
      ...getNavigationPropsV2({
        currentRiver,
        title,
        category: "nav_bar",
      }),
      setToggleMenu,
      setCloseMenu,
    }),
    [currentRiver, setCloseMenu, setToggleMenu]
  );

  const NavBar = React.useMemo(
    () =>
      getNavigationPluginModule(
        "nav_bar",
        currentRiver,
        plugins
      ) as React.ComponentType<MobileNavBarPluginProps>,
    []
  );

  const { screenData, pathname } = routeData;

  const offlinePlugin = getComponentModule({
    plugins,
    components: {},
    componentType: "offline-experience",
  });

  const { OfflineFallbackScreen } = offlinePlugin;

  const style = React.useMemo(
    () => ({
      flex: 1,
      backgroundColor: isValidColor(backgroundColor)
        ? backgroundColor
        : theme.app_background_color,
      paddingTop: 0,
      // paddingBottom: insets.bottom, // Should be applied directly by UI plugin
      // paddingLeft: insets.left, // Should be applied directly by UI plugin
      // paddingRight: insets.right, // Should be applied directly by UI plugin
    }),
    [insets, screenWidth, theme.app_background_color, backgroundColor]
  );

  const targetScreenData =
    currentScreenData || (screenData as any)?.targetScreen || screenData;

  useScreenOrientationHandler({ screenData: targetScreenData });

  return (
    <ScreenContextProvider pathname={pathname}>
      <View style={style}>
        <NavBar
          {...{ ...(navBarProps as MobileNavBarPluginProps), id: pathname }}
        />

        <OfflineFallbackScreen screenData={_screenData}>
          {/* @TODO RouteManager doesn't use props, can they be removed ?  */}
          <RouteManager pathname={pathname} screenData={_screenData} />
        </OfflineFallbackScreen>
      </View>
    </ScreenContextProvider>
  );
}
