import React from "react";
import {
  Animated,
  useWindowDimensions,
  ViewProps,
  ViewStyle,
} from "react-native";
import { useSafeAreaFrame } from "react-native-safe-area-context";
import { PathnameContext } from "../../Contexts/PathnameContext";

type Props = {
  children: any;
  style: any;
  pointerEvents: ViewProps["pointerEvents"];
  overlayStyle: ViewStyle;
  animating: boolean;
  contentStyle: ViewStyle;
  pathname: string;
};

/**
 * Animated View container for any purpose.
 * Renders given children with given style object.
 * Optional property: pointer event (e.g. to prevent interaction with animating scene).
 * @param {React.Component} children Just about anything
 * @param {object} style             Container style
 * @param {} pointerEvents           box-none|none|box-only|auto (default)
 * @param overlayStyle
 * @param animating
 * @param contentStyle
 * @param screenUniqueId
 */
export function Scene({
  children,
  style,
  pointerEvents = "auto",
  overlayStyle,
  animating,
  contentStyle,
  pathname,
}: Props) {
  const frame = useSafeAreaFrame();
  const { height, width } = useWindowDimensions();

  if (overlayStyle && animating) {
    // Safe frame takes too long to update,
    // so we can not rely on it there, and use height and width instead.
    return (
      <Animated.View
        pointerEvents={pointerEvents}
        style={[style, contentStyle, { height, width }]}
      >
        <PathnameContext.Provider value={pathname}>
          {children}
        </PathnameContext.Provider>
        <Animated.View style={overlayStyle} />
      </Animated.View>
    );
  }

  return (
    <Animated.View
      pointerEvents={pointerEvents}
      style={[style, contentStyle, frame]}
    >
      <PathnameContext.Provider value={pathname}>
        {children}
      </PathnameContext.Provider>
    </Animated.View>
  );
}
