import * as React from "react";
import {
  Dimensions,
  Platform,
  StyleSheet,
  View,
  ViewStyle,
} from "react-native";
import { Edge, SafeAreaView } from "react-native-safe-area-context";

import { isTV } from "@applicaster/zapp-react-native-utils/reactUtils";
import { useIsTablet } from "@applicaster/zapp-react-native-utils/reactHooks";
import { PlayerDetails } from "./PlayerDetails";
import { playerDimesionsHack } from "./utils";
import { PathnameContext } from "../../Contexts/PathnameContext";
import { ScreenContextProvider } from "../../Contexts/ScreenContext";

type DimensionsT = {
  width: number | string;
  height: number | string;
  aspectRatio?: number;
};

type Configuration = {
  [key: string]: any;
  tablet_landscape_sidebar_width?: string;
  tablet_landscape_player_container_background_color?: string;
};

type Props = {
  entry: ZappEntry;
  style: ViewStyle & DimensionsT;
  containerStyle?: ViewStyle;
  inline?: boolean;
  docked?: boolean;
  pip?: boolean;
  isModal?: boolean;
  fullscreen?: boolean;
  isTabletPortrait?: boolean;
  children: (playerDimensions: DimensionsT) => React.ReactNode;
  configuration: Configuration;
};

const defaultStyles = StyleSheet.create({
  playerContainer: { position: "relative", alignSelf: "center", zIndex: 200 },
  playerDetails: { flex: 1, paddingTop: 20 },
});

const orientationStyles = StyleSheet.create({
  landscape: { flexDirection: "row" },
  portrait: { flexDirection: "column" },
});

const directionStyles = (isTabletLandscape: boolean): ViewStyle => {
  if (isTabletLandscape) {
    return orientationStyles.landscape;
  }

  return orientationStyles.portrait;
};

const getScreenAspectRatio = () => {
  const { width, height } = Dimensions.get("screen");

  const longEdge = Math.max(width, height);
  const shortEdge = Math.min(width, height);

  return longEdge / shortEdge;
};

const safeAreaStyles = (
  configuration: Configuration,
  isTablet: boolean
): ViewStyle => {
  const tablet_landscape_player_container_background_color =
    configuration?.tablet_landscape_player_container_background_color;

  return {
    backgroundColor: isTablet
      ? tablet_landscape_player_container_background_color
      : "transparent",
  };
};

const getEdges = (
  isTablet: boolean,
  docked: boolean,
  isInlineModal: boolean
) => {
  if (isTablet) {
    return docked ? ["right", "left"] : ["top"];
  }

  // mobile
  if (isInlineModal && docked) {
    return ["bottom"];
  }

  if (!isInlineModal && Platform.OS === "android") {
    return [];
  }

  return ["top"];
};

const getTabletWidth = (
  configuration: Configuration,
  dimensions: DimensionsT
) => {
  const tablet_landscape_sidebar_width =
    configuration?.tablet_landscape_sidebar_width;

  const { width } = dimensions;

  const sidebarWidth = Number(tablet_landscape_sidebar_width?.replace("%", ""));

  if (tablet_landscape_sidebar_width?.includes("%")) {
    return Number(width) * (1 - sidebarWidth / 100);
  }

  if (Number.isNaN(sidebarWidth)) {
    return Number(width) * 0.65;
  }

  return Number(width) - sidebarWidth;
};

export const PlayerWrapper = (props: Props) => {
  const {
    entry,
    style,
    containerStyle,
    inline,
    docked,
    isModal,
    children,
    isTabletPortrait,
    configuration,
    pip,
  } = props;

  const isTablet = useIsTablet();

  const isInlineModal = inline && isModal;

  const isTabletLandscape = !isTV() && isTablet && !isTabletPortrait;

  const tabletWidth = getTabletWidth(configuration, style);

  const baseDimensions: DimensionsT = React.useMemo(
    () => ({
      width:
        isInlineModal && !docked && isTabletLandscape ? tabletWidth : "100%",
      height: isInlineModal && docked ? style.height : undefined,
    }),
    [isInlineModal, tabletWidth, style.height, docked]
  );

  const playerDimensions: DimensionsT = React.useMemo(
    () => ({
      ...baseDimensions,
      width: isInlineModal && docked ? undefined : baseDimensions.width,
      aspectRatio: !isInlineModal && !pip ? getScreenAspectRatio() : 16 / 9,
    }),
    [baseDimensions, isInlineModal, pip]
  );

  const containerDimensions: DimensionsT = React.useMemo(
    () => ({
      ...baseDimensions,
      aspectRatio:
        isInlineModal && docked ? undefined : playerDimensions.aspectRatio,
    }),
    [baseDimensions, isInlineModal, docked, playerDimensions.aspectRatio]
  );

  const WrapperView = React.useMemo(() => (isTV() ? View : SafeAreaView), []);

  const childrenStyles = React.useMemo(
    () => ({ ...playerDimensions, ...playerDimesionsHack }),
    [containerDimensions, playerDimesionsHack]
  );

  return (
    <WrapperView
      edges={getEdges(isTablet, docked, isInlineModal) as readonly Edge[]}
      style={[
        directionStyles(isTabletLandscape),
        safeAreaStyles(configuration, isTablet),
        style,
        playerDimesionsHack,
      ]}
    >
      <ScreenContextProvider pathname={"videoModal"}>
        <PathnameContext.Provider value={"videoModal"}>
          <View
            testID={`${entry?.id}-player-container`}
            style={[
              defaultStyles.playerContainer,
              containerDimensions,
              containerStyle,
              playerDimesionsHack,
            ]}
          >
            {children(childrenStyles)}
          </View>
          {isInlineModal && !docked ? (
            <PlayerDetails
              configuration={configuration}
              style={defaultStyles.playerDetails}
              entry={entry}
              isTabletLandscape={isTabletLandscape}
            />
          ) : (
            <View style={defaultStyles.playerDetails} />
          )}
        </PathnameContext.Provider>
      </ScreenContextProvider>
    </WrapperView>
  );
};
