import React, { createContext, ReactNode, useMemo, useState } from "react";
import {
  useCurrentScreenData,
  useNavigation,
  useRoute,
  isNavBarVisible,
} from "@applicaster/zapp-react-native-utils/reactHooks";
import { useModalNavigationContext } from "@applicaster/zapp-react-native-ui-components/Contexts/ModalNavigationContext";
import { useNestedNavigationContext } from "@applicaster/zapp-react-native-ui-components/Contexts/NestedNavigationContext";

type ScreenContextType = {
  navBar: {
    visible: boolean;
    title: string;
    summary: string;
    setVisible: (state: boolean) => void;
    setTitle: (title: string) => void;
    setSummary: (subtitle: string) => void;
  };
  legacyFormatScreenData: LegacyNavigationScreenData | null;
};

export const ScreenContext = createContext<ScreenContextType>({
  navBar: {
    visible: true,
    title: "",
    summary: "",
    setVisible: (_state) => {},
    setTitle: (_title) => {},
    setSummary: (_subtitle) => {},
  },
  legacyFormatScreenData: {} as LegacyNavigationScreenData,
});

export function ScreenContextProvider({
  children,
  pathname,
}: {
  children: ReactNode;
  pathname: string;
}) {
  const { data, modalData, videoModalState, canGoBack } = useNavigation();

  // There are 4 route scenarios
  // For regular screens take date from navigator stack.
  // is path is model grab screenData from modal state
  // if path is video modal grab screenData from video state
  // if path is hook modal grab screenData from hook state

  const { screenData: routeScreenData } = useRoute();

  const isModal = useModalNavigationContext();
  const isNested = useNestedNavigationContext();
  const currentScreenScreenData = useCurrentScreenData();

  const [navBarTitle, setNavBarTitle] = useState("");
  const [navBarSummary, setNavBarSummary] = useState("");
  const [navBarVisible, setNavBarVisible] = useState(true);

  const isHook = pathname.includes("hook");

  // screenData is replacement of useScreenContext()
  const screenData = useMemo(() => {
    if (isModal && modalData) return modalData ?? ({} as NavigationScreenData);

    if (isNested && data?.nested) {
      return data?.nested ?? ({} as ScreenData);
    }

    return data ?? ({} as ScreenData);
  }, [isModal, isNested, modalData, data]);

  const isVisible = isNavBarVisible(
    pathname,
    currentScreenScreenData, // what fallback should be if currentScreenData is not available?
    navBarVisible,
    canGoBack(isHook, pathname),
    videoModalState
  );

  const navBarState = useMemo(() => {
    return {
      title: navBarTitle,
      summary: navBarSummary,
      visible: isVisible,
      setTitle: setNavBarTitle,
      setSummary: setNavBarSummary,
      setVisible: setNavBarVisible,
    };
  }, [isVisible, navBarTitle, navBarSummary]);

  return (
    <ScreenContext.Provider
      value={useMemo(
        () => ({
          navBar: navBarState,
          legacyFormatScreenData: routeScreenData,
        }),
        [navBarState, screenData, routeScreenData]
      )}
    >
      {children}
    </ScreenContext.Provider>
  );
}

export function withScreenContext(Component: React.ComponentType<any>) {
  return function WithScreenContextWrapper(props) {
    return (
      <ScreenContext.Consumer>
        {(value) => <Component {...props} screenContext={value} />}
      </ScreenContext.Consumer>
    );
  };
}
